"""
	Functions for drawing a board.
"""

__author__ = "Roland Lezuo <roland.lezuo@chello.at>"
__licence__ = "GNU General Public License Version 2"


import logging
logger = logging.getLogger("board")


import gtk
import pango
import gettext
import array

# graphical representation of the board
# draws onto a gtk.drawable
def get_size(size, scale):
	s = int(Board._sidelength * scale)
	if s % 2 == 0:
		s += 1
	return ((size+1)*s, (size+1)*s) 

class Board:
	_sidelength = 21	#XXX
	def __init__(self, drawable, game, size, scale, multiplayer, movenumbers, showcoords, pango_ctx, installed_prefix):
		self.drawable = drawable
		self.size = size
		self.mark_x = None
		self.in_putting = False
		self.gameover = False
		self.multiplayer = multiplayer
		self._movenumbers = movenumbers
		#self._passareas = passareas
		self._showcoords = showcoords
		self.installed_prefix = installed_prefix
		self.statusbar = None

		self.pango_context = pango_ctx
		self._marked = []


		self.set_scale(scale)	# call before setting cursor!
		
		self._result = None

		# background image, if any, set it using set_background_image
		self.img_board = None
		self.background_color = gtk.gdk.Color(0xFFFF, 0xFFFF, 0xFFFF)
		self.game = game
	def set_background_image(self, img):
		try:
			self.img_board = gtk.gdk.pixbuf_new_from_file(img)
		except:
			self.img_board = None
			pass
	def set_background_color(self, c):
		self.background_color = c


	def set_movenumbers(self, mn):
		self._movenumbers = mn

	def set_showcoords(self, sc):
		self._showcoords = sc

	def set_scale(self, scale):
		# self.sidelength needs to be odd!
		self.scale = scale
		self.sidelength = int(self._sidelength * scale)
		if self.sidelength % 2 == 0:
			self.sidelength += 1
		self.starpointsize = int(2 * scale)
		self.image_size = int(self._sidelength * self.scale)

		s = self.image_size 
		self.img_black = gtk.gdk.pixbuf_new_from_file_at_size(self.installed_prefix + 'black_stone.svg', s, s)
		self.img_white = gtk.gdk.pixbuf_new_from_file_at_size(self.installed_prefix + 'white_stone.svg', s, s)

		#FIXME, make svg out of theese too
		self.img_board_black = gtk.gdk.pixbuf_new_from_file(self.installed_prefix + 'board_black.png').scale_simple(s, s, gtk.gdk.INTERP_HYPER)
		self.img_board_white = gtk.gdk.pixbuf_new_from_file(self.installed_prefix + 'board_white.png').scale_simple(s, s, gtk.gdk.INTERP_HYPER)

		self.setcursor_black = gtk.gdk.Cursor(gtk.gdk.display_get_default(), self.img_black, int(s/2), int(s/2))
		self.setcursor_white = gtk.gdk.Cursor(gtk.gdk.display_get_default(), self.img_white, int(s/2), int(s/2))

	def set_default_cursor(self):
		self.drawable.set_cursor(None)
	def set_black_cursor(self):
		self.drawable.set_cursor(self.setcursor_black)
	def set_white_cursor(self):
		self.drawable.set_cursor(self.setcursor_white)
	def get_size(self):
		return get_size(self.size, self.scale)
	# get offsets into the free areas of the board where one can draw whatever he wants
	def get_header_offset(self):
		return (self.sidelength + self.sidelength/3, self.sidelength/3)
	def get_footer_offset(self):
		return (self.sidelength + self.sidelength/3, self.sidelength*self.size+ self.sidelength/3)

	def redraw(self):
		gc = self.drawable.new_gc()
		ss = self.sidelength
		drawable = gtk.gdk.Pixmap(self.drawable, (self.size+1)* ss, (self.size+1)*ss)	

		# draw as many backgroudnimage tiles as necessary, if we want a background at all
		if self.img_board:
			x = 0
			while x < self.size * ss + ss:
				y = 0
				while y < self.size * ss + ss:
					drawable.draw_pixbuf(gc, self.img_board, 0, 0, x, y)
					y += self.img_board.get_height()
				x += self.img_board.get_width()
		else:
			# clear the background with given color
			gc.set_rgb_fg_color(self.background_color)
			drawable.draw_rectangle(gc, True, 0, 0, (self.size+1)*ss, (self.size+1)*ss)

		# draw all elements in black
		gc.set_rgb_fg_color(gtk.gdk.Color(0x0,0x0,0x0))
	
		for x in range(self.size):
			drawable.draw_line(gc, x * ss + ss , ss, x * ss + ss, self.size * ss)
		for y in range(self.size):
			drawable.draw_line(gc, ss, y * ss + ss, self.size * ss, y * ss + ss)

		# draw coordinates
		black = gtk.gdk.Color(0,0,0)
		if self.pango_context and self._showcoords:
			ss = self.sidelength
			for i in range(self.size):
				text = pango.Layout(self.pango_context)
				text.set_text(repr(self.size-i))
				text.set_width(ss* pango.SCALE)
				textCh = pango.Layout(self.pango_context)
				if i>=8:
					textCh.set_text(chr(ord('A')+i+1)) # there is no I (spoon)
				else :
					textCh.set_text(chr(ord('A')+i)) # there is no I (spoon)
				textCh.set_width(ss* pango.SCALE)
				xa,ya = (ss/2+ss*i,2);
				xe,ye = (ss/2+ss*i,ss*self.size);
				t1,(x1,y1,x2,y2) = text.get_pixel_extents()
				w = x2-x1
				h = y2-y1
				drawable.draw_layout(gc, ya+ss/2-w/2, xa+ss/2-h/2, text, black)		
				drawable.draw_layout(gc, ye+ss/2-w/2, xe+ss/2-h/2, text, black)		
				drawable.draw_layout(gc, xe+ss/2-w/2, ye+ss/2-h/2, textCh, black)		
				drawable.draw_layout(gc, xa+ss/2-w/2, ya+ss/2-h/2, textCh, black)		

		# the starpoints
		if self.size == 9:
			starpoints = [ (2,2), (6,2), (4,4), (2,6), (6,6) ]
		elif self.size == 13:
			starpoints = [ (3,3), (3,6), (3,9), (6,3), (6,6), (6,9), (9,3), (9,6), (9,9) ]
		elif self.size == 19:
			starpoints = [ (3,3), (3,9), (3,15), (9,3), (9,9), (9,15), (15,3), (15,9), (15,15) ]
		else:
			starpoints = [] 	#do not know, but this size is not supported atm anyway.

		sps = self.starpointsize
		for p in starpoints:
			x,y = p
			drawable.draw_rectangle(gc, True, ss + x*ss - sps, ss + y*ss - sps, 2*sps + 1, 2*sps + 1)

		# ohter drawing if the game has ended
		if self.gameover:
			self._draw_gameover(drawable, gc)

		else:
			self._draw_stones(drawable, gc)
			# additionally add marks to stones
			for m in self._marked:
				self._mark_stone(gc, drawable, m)

		self.drawable.draw_drawable(gc, drawable, 0, 0, 0, 0, -1, -1)

	def _draw_stones(self, drawable, gc):
		ss = self.sidelength

#		if self._passareas:
#			# and mark the corners, where passing is possible
#			drawable.draw_pixbuf(gc, self.img_board_black, 0, 0, 0, 0)
#			drawable.draw_pixbuf(gc, self.img_board_black, 0, 0, 0, ss*self.size)
#			drawable.draw_pixbuf(gc, self.img_board_black, 0, 0, ss*self.size, 0)
#			drawable.draw_pixbuf(gc, self.img_board_black, 0, 0, ss*self.size, ss*self.size)

		# and draw the stones on the board
		white_stones = self.game.get_white_stones()
		black_stones = self.game.get_black_stones()
		moves = self.game.get_move_history()

		for ws in white_stones:
			x,y = self._xycoords_from_stonecoord(ws)
			if x == -1: continue	# pass move
			drawable.draw_pixbuf(gc, self.img_white, 0, 0, x, y)
			if self._movenumbers:
				self._draw_numbers(drawable, ws, moves, 'w')
		for bs in black_stones:
			x,y = self._xycoords_from_stonecoord(bs)
			if x == -1: continue	# pass move
			drawable.draw_pixbuf(gc, self.img_black, 0, 0, x, y)
			if self._movenumbers:
				self._draw_numbers(drawable, bs, moves, 'b')

		caps = self.game.get_captures()
		self.update_statusbar(gettext.gettext('white: ') +  ' ' + caps)


	# move_history is a dictionary with filed names as keys, and a list of int as values
	def _draw_numbers(self, drawable, pos, move_history, color):
		# only draw if we have a pango context
		if not self.pango_context:
			logger.warning("Attempt to draw move numbers without pange context")
			return

		if color == 'w':
			color = gtk.gdk.Color(0,0,0)
		else:
			color = gtk.gdk.Color(0xFFFF, 0xFFFF, 0xFFFF)

		if not move_history.has_key(pos):
			return
		
		numbers = move_history[pos]
		# do not print more than 5 numbers per field
		if len(numbers) > 3:
			si = -3
			t = '...'
		else:
			si = 0
			t = ''
		
		for i in numbers[si:]:
			t += str(i)
			t += ' '
		t = t[:-1]

		ss = self.sidelength
		text = pango.Layout(self.pango_context)
		text.set_text(t)
		text.set_width(ss* pango.SCALE)

		x,y = self._xycoords_from_stonecoord(pos)
		t1,(x1,y1,x2,y2) = text.get_pixel_extents()
		w = x2-x1
		h = y2-y1
		gc = self.drawable.new_gc()
		drawable.draw_layout(gc, x+ss/2-w/2, y+ss/2-h/2, text, color)

		
	# draw end result
	def _draw_gameover(self, drawable, gc):
		# chache this for performance
		if not self._result:
			self._result = self.game.get_final_status()
		white_stones = self.game.get_white_stones()
		black_stones = self.game.get_black_stones()


		# draw the areas
		for wt in self._result['white_territory']:
			x,y = self._xycoords_from_stonecoord(wt)
			drawable.draw_pixbuf(gc, self.img_board_white, 0, 0, x, y)
		for bt in self._result['black_territory']:
			x,y = self._xycoords_from_stonecoord(bt)
			drawable.draw_pixbuf(gc, self.img_board_black, 0, 0, x, y)
		# draw the stones, in there "normal" color if alive, and transparent if not, other states are not drawn at all
		for ws in white_stones:
			x,y = self._xycoords_from_stonecoord(ws)
			if ws in self._result['alive']:
				drawable.draw_pixbuf(gc, self.img_white, 0, 0, x, y)
			elif ws in self._result['dead']:
				drawable.draw_pixbuf(gc, self.img_white, 0, 0, x, y)
				drawable.draw_pixbuf(gc, self.img_board_black, 0, 0, x, y)
			else:
				pass
		for bs in black_stones:
			x,y = self._xycoords_from_stonecoord(bs)
			if bs in self._result['alive']:
				drawable.draw_pixbuf(gc, self.img_black, 0, 0, x, y)
			elif bs in self._result['dead']:
				drawable.draw_pixbuf(gc, self.img_black, 0, 0, x, y)
				drawable.draw_pixbuf(gc, self.img_board_white, 0, 0, x, y)
			else:
				pass

	# marks a stone with a rectangle
	def _mark_stone(self, gc, drawable, where):
		# a red square, good to be seen on white stones
		gc.set_rgb_fg_color(gtk.gdk.Color(0x8000,0x0000,0x0000))
		x,y = self._xycoords_from_stonecoord(where)
		im2 = self.image_size/2
		xl = x + im2/2
		yl = y + im2/2
		drawable.draw_rectangle(gc, False,  xl, yl, im2, im2)

		# a white square, good to be seen on black stones
		gc.set_rgb_fg_color(gtk.gdk.Color(0x8000,0x8000,0x8000))
		x,y = self._xycoords_from_stonecoord(where)
		im2 = self.image_size/2-2
		xl = x + im2/2 + 2
		yl = y + im2/2 + 2
		drawable.draw_rectangle(gc, False,  xl, yl, im2, im2)

	# add given stone to list of stones to mark
	def mark_stone(self, stone):
		self._marked.append(stone)
	# clear list of marked stones
	def clear_marks(self):
		self._marked = []

	# returns (x,y) screen coordinates from logical coordinates
	def _xycoords_from_stonecoord(self, (xl,yl)):
		if xl == -1: return (-1,-1)	# a pass move
		xs = xl * self.sidelength + self.sidelength - self.image_size/2
		ys = yl * self.sidelength + self.sidelength - self.image_size/2
		ymax = self.sidelength * self.size 
		return (xs, ymax-ys)

	# int x, int y coordinates of mouse click  returns logical coordinates of field clicked, 
	# or (-1,-1) if PASS or None if hit nothing
	def get_stone_coordinates(self, x, y):
		# calculate where to put the stone
		# get rid of offset
		x = x - self.sidelength
		y = (self.sidelength* self.size )- y 
	
		sh = float(self.sidelength) / 2.0
		x_click = int(round(float(x) / sh))
		y_click = int(round(float(y) / sh))

		if (x_click <= -1 or x_click >= self.size*2-1) and (y_click <= -1 or y_click >= self.size*2-1):
			# one of the PASS fields
			return (-1, -1)
		else:
			# let there be nonclickable space between the fields
			if x_click % 2: return None
			if y_click % 2: return None
		
			x_click = 1 + x_click / 2
			y_click = 1 + y_click / 2

			# outside the field
			if x_click > self.size: return None
			if y_click > self.size: return None

			return (x_click, y_click)

	# changes how the board is drawn
	def game_is_over(self):
		self.gameover = True
	# someone realized another defensive move would have been necessary :)
	def undo_game_is_over(self):
		self.gameover = False
		if self._result:
			self._result = None
		
	def set_statusbar(self, sb):
		self.statusbar = sb
	def update_statusbar(self, txt):
		if self.statusbar != None:
			self.statusbar.push(1, txt)
