# a stupid wrapper doing all gconf stuff
# gnomego consists of a whole bunch of applications which all share this

import gconf

class GnomeGoPref:
	# these values will be set correctly when installing the package
	nameofthegame_default = '/usr/games/gnugo --mode gtp'
	installed_prefix = '/usr/share/gnomego/'
	gnomego_version = '0.5.1'

	# stored in gconf
	keys = { 
		'size' : '/apps/gnomego/board_size',
		'cmd' : '/apps/gnomego/gnugo_cmd',
		'komi' : '/apps/gnomego/gnugo_komi',
		'scale' : '/apps/gnomego/board_scale',
		'level' : '/apps/gnomego/gnugo_level',
		'boardimg' : '/apps/gnomego/board_img',
		'boardcolor' : '/apps/gnomego/board_color',
		'vshuman' : '/apps/gnomego/game_vshuman',
		'movenumber' : '/apps/gnomego/game_movenumber',
		'showcoords' : '/apps/gnomego/game_showcoords',
		'handicap' : '/apps/gnomego/handicap',
		'gnugo_is_black' : '/apps/gnomego/gnugo_is_black',
		'sgfpath' : '/apps/gnomego/screensaver_path',
		'version' : '/apps/gnomego/version'
	}

	def __init__(self):
		self._gcc = gconf.client_get_default()
		
		# this is done to check for illeagal values stored in gconf,
		# this at least happened for gnomego 0.4 -> gnomego 0.5 an int changed to float
		cmd = self.get_string('cmd')
		size = self.get_int('size')
		scale = self.get_float('scale')
		level = self.get_int('level')
		komi = self.get_float('komi')
		human = self.get_bool('vshuman')
		moven = self.get_bool('movenumber')
		showcoords = self.get_bool('showcoords')
		boardimg = self.get_string('boardimg')
		boardcolor = self.get_string('boardcolor')
		sgfpath = self.get_string('sgfpath')
		handicap = self.get_int('handicap')
		version = self.get_string('version')
		if handicap == None: 
			handicap = 0
		gnugo_is_black = self.get_bool('gnugo_is_black')
		if gnugo_is_black == None: 
			gnugo_is_black = False

		#check the values and write back sane defaults if needed
		if not komi:
			# we read a 0.0 if komi is not set, so we can't distinguish that from the setting 0.0
			# therefore a 0.0 komi will be stored as 1.0 ...
			# default komi is 6.5
			self.set_float('komi', 7.5)
		if not boardcolor:
			self.set_string('boardcolor', '#FFFFFF')
		if not scale: 
			self.set_float('scale', 1.0)
		if not size:
			self.set_int('size', 19)
		if not boardimg:
			self.set_string('boardimg', self.installed_prefix + 'board1.png')
		if not sgfpath:
			self.set_string('sgfpath', self.installed_prefix + 'sgfs')
		if not version:
			self.set_string('version', self.gnomego_version)

	def get_string(self, key):
		return self._get("get_string", key)
	def get_int(self, key):
		return self._get("get_int", key)
	def get_float(self, key):
		return self._get("get_float", key)
	def get_bool(self, key):
		return self._get("get_bool", key)

	def _get(self, getter, key):
		if not key in self.keys:
			raise "Unknown GnomeGoPref key:" + str(key)
		val = None
		try:
			val = type(self._gcc).__dict__[getter](self._gcc, self.keys[key])
		except:
			print "GnomeGoPref: exception reading " + str(key) +  "...deleting key!"
			self._gcc.remove_dir(key)
		#print str(key) + ":=" + str(val)
		return val

	def set_string(self, key, val):
		self._set("set_string", key, val)
	def set_int(self, key, val):
		self._set("set_int", key, val)
	def set_float(self, key, val):
		self._set("set_float", key, val)
	def set_bool(self, key, val):
		self._set("set_bool", key, val)

	def _set(self, setter, key, val):
		if not key in self.keys:
			raise "Unknown GnomeGoPref key:" + str(key)
		try:
			type(self._gcc).__dict__[setter](self._gcc, self.keys[key], val)
		except:
			print "GnomeGoPref: exception writing " + str(key) +  "...deleting key!"
			self._gcc.remove_dir(key)
		#print str(key) + ":=" + str(val)
	
if __name__ == '__main__':
	p = GnomeGoPref()
